// Copyright 2020 The Android Open Source Project
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#ifndef _HW_ANDROID_PIPE_BASE_H
#define _HW_ANDROID_PIPE_BASE_H

// vtbl in Android pipe.
typedef struct AndroidPipeHwFuncs {
    void (*closeFromHost)(void* hwpipe);
    void (*signalWake)(void* hwpipe, unsigned flags);
    int (*getPipeId)(void* hwpipe);
} AndroidPipeHwFuncs;

#endif /* _HW_ANDROID_PIPE_BASE_H */
