// Copyright (C) 2022 The Android Open Source Project
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#pragma once

#include <cstdint>

namespace android {
namespace base {

// Get memory statistics
struct MemUsage {
    uint64_t resident;
    uint64_t resident_max;
    uint64_t virt;
    uint64_t virt_max;
    uint64_t total_phys_memory;
    uint64_t avail_phys_memory;
    uint64_t total_page_file;
};
MemUsage getMemUsage();

// Known distinct kinds of disks.
enum class DiskKind {
    Hdd,
    Ssd
};

} // namespace base
} // namespace android
